"""
Embeddings integration module

This module provides embeddings integrations and factory.
"""

from .factory import EmbedderFactory

EmbedderFactory = EmbedderFactory

__all__ = [
    "EmbedderFactory",
]
